#!/bin/bash

# Create screenshots of all entries listed in LIST_FILE and upload the result
# to a webserver (http-post to php-script).
#
# Last change: 2008-05-11 (Frank Baettermann)

UPLOAD_URL='https://your-domain.tld/osmshot-upload.php'
UPLOAD_SECRET='YOUR-SECRET'
UPLOAD_SUBDIR='screenshots'

# ---- EOF static config ----

# help
if [ $# -eq 0 ]; then
    echo "Usage: $0 [OPTIONS]"
    echo "  --list FILE     List of screeshots"
    echo "                  Format:"
    echo "                  NAME (space) X1,Y1,X2,Y2,ZOOM"
    exit 1
fi
				    
# defaults
LIST_FILE=""
DATE=`date +%Y%m%d`
LIST_FILETMP="${LIST_FILE}.tmp"
STATS_TOTAL=0
STATS_SUCCESS=0

# parse parameters
while [ $# -gt 0 ]; do
    case "$1" in
        "--list")
            LIST_FILE=$2
            shift
            ;;
        *)
            echo "Unknown parameter: $1"
            exit 1
            ;;
    esac
shift
done

# check settings
if [ -z "$LIST_FILE" ]; then
    echo "Error: No list supplied."
    exit 1
elif ! [ -r "$LIST_FILE" ]; then
    echo "Error: List \"`pwd`/${LIST_FILE}\" cannot be read. Check permissions."
    exit 1
fi

# check lockfile
LOCKFILE="${0}.`basename "$LIST_FILE"`.lock"
if [ -f $LOCKFILE ]; then
    OLDPID=`cat $LOCKFILE`
    if [ `ps -p $OLDPID | wc -l` -eq 2 ]; then
        # kill old process as it hangs (most likely)
	echo -n "Killing old process (`cat $LOCKFILE`)..."
        kill -9 `cat $LOCKFILE`
        if [ $? -ne 0 ]; then
	    echo "Error"
	    exit 1
	fi
	echo "Ok"
    else
	echo "Last run (`stat -c %x $LOCKFILE | cut -d '.' -f 1`) did not finish tidily."
    fi
fi
# create lockfile
echo $$ > $LOCKFILE

# remove comments and sqeeze spaces and tabs
grep -v ^# "$LIST_FILE" | tr "\t" " " | tr -s " " > "$LIST_FILETMP"
# remove empty lines
sed '/^[ \t]*$/d' "$LIST_FILETMP" >  "${LIST_FILETMP}2"
# delete second tmp-file
rm "${LIST_FILETMP}"
mv "${LIST_FILETMP}2"  "${LIST_FILETMP}"

# process all entries
while read LINE; do
    SHOT_NAME=`echo $LINE | cut -d ' ' -f 1`
    SHOT_TILES=`echo $LINE | cut -d ' ' -f 2 | cut -d ',' -f 1-4`
    SHOT_ZOOM=`echo $LINE | cut -d ' ' -f 2 | cut -d ',' -f 5`
    SHOT_DIR="`pwd`/${SHOT_NAME}/${SHOT_ZOOM}"
    echo "${SHOT_NAME} (zoom ${SHOT_ZOOM}):"
    
    CANCEL=0 # 1 = cancel next steps
    
    echo -n "    Create directory (${SHOT_DIR})... "
    mkdir -p "$SHOT_DIR"
    if [ $? -ne 0 ]; then
	echo "Error"
	CANCEL=1
    fi
    
    if [ $CANCEL -eq 0 ]; then
	echo "Ok"
        echo -n "    Retrieve tiles (${SHOT_TILES})... "
	./mdl.sh \
	    --tiles ${SHOT_TILES},${SHOT_ZOOM} \
	    --montage \
	    --output-dir "${SHOT_DIR}/${DATE}" \
	    &> /dev/null
        if [ $? -ne 0 ]; then
    	    echo "Error"
    	    CANCEL=1
	fi
    fi

    if [ $CANCEL -eq 0 ]; then
	echo "Ok"
        echo -n "    Copy montage-result (`stat -c %s ${SHOT_DIR}/${DATE}/montage.png` bytes)... "
	mv "${SHOT_DIR}/${DATE}/montage.png" "${SHOT_DIR}/${DATE}.png"
        if [ $? -ne 0 ]; then
    	    echo "Error"
    	    CANCEL=1
	fi
    fi

    if [ $CANCEL -eq 0 ]; then
	# delete tiles
	rm -r "${SHOT_DIR}/${DATE}"	
	echo "Ok"
	
	LOG="${SHOT_DIR}/curl-upload.${DATE}.log"
        echo -n "    Uploading... "
	curl \
	    -F "file=@${SHOT_DIR}/${DATE}.png" \
	    -F "directory=${UPLOAD_SUBDIR}/${SHOT_NAME}/${SHOT_ZOOM}" \
	    -F "secret=${UPLOAD_SECRET}" \
	    "${UPLOAD_URL}" \
	    &> $LOG

        if [ $? -ne 0 ] || [ `grep ^Ok "${LOG}" | wc -l` -ne 1 ]; then
    	    echo "Error"
    	    CANCEL=1
	fi
	if [ $CANCEL -eq 0 ]; then
#	    rm "${SHOT_DIR}/curl-upload.${DATE}.log"
	    echo "Ok"
	fi
    fi

    # distance between entries
    echo ""
    
    # count
    ((STATS_TOTAL++))
    if [ $CANCEL -eq 0 ]; then
	((STATS_SUCCESS++))
    fi
done < "$LIST_FILETMP"

# delete tmp-file
rm "$LIST_FILETMP"
rm "$LOCKFILE"

# show stats

echo "---- Done! ----"
echo "Total: $STATS_TOTAL"
echo "Errorneous: $(($STATS_TOTAL - $STATS_SUCCESS))"
